import java.applet.*;
import java.awt.*;

public class EX15E extends Applet implements Runnable
{
	protected final int ImageId = 1;
	protected final int ImageWidth = 1000;
	protected final int ImageDisplayWidth = 300;
	protected final int ImageDisplayHeight = 100;
	protected final int ImageXDecrement = 2;
	protected final int ImageDelay = 20;

	protected Image SingleImage;
	protected boolean ImageLoaded = false;
	protected int CurrentDisplayXValue;
	protected MediaTracker ImageTracker;
	protected Thread ImageThread;
	protected Label ProgressMessage = new Label("Loading Image");
	protected Image OffScreenImage;


	public void init()
	{
        // If you use a ResourceWizard-generated "control creator" class to
        // arrange controls in your applet, you may want to call its
        // CreateControls() method from within this method. Remove the following
        // call to resize() before adding the call to CreateControls();
        // CreateControls() does its own resizing.
        //----------------------------------------------------------------------
		resize(320, 240);

		add(ProgressMessage);

		// create a tracker to monitor image loading
		ImageTracker = new MediaTracker(this);

		// add the image to be tracked
		SingleImage = getImage(getCodeBase(), 
				"images\\FullBanner.gif");
		ImageTracker.addImage(SingleImage, ImageId);
	}

	public void paint(Graphics g)
	{
		// if haven't loaded the image, then don't do anything
		if (ImageLoaded) {
			g.clipRect(0, 0, ImageDisplayWidth, ImageDisplayHeight);

			// draw the first round of the image
			g.drawImage(SingleImage, CurrentDisplayXValue, 0, this);

			// calculate where the wrapping image starts
			int wrapXLocation = CurrentDisplayXValue + ImageWidth;

			// if the wrap image has started into the displayable
			//   range, draw the image again
			if (wrapXLocation < ImageDisplayWidth)
				g.drawImage(SingleImage, wrapXLocation, 0, this);
		}
	}

	public void update(Graphics g)
	{
		// create an off screen image if don't have one
		if (OffScreenImage == null)
			OffScreenImage = createImage(size().width, size().height);

		// get a grahpics that can be used with the off screen image
		Graphics offScreenGraphics = OffScreenImage.getGraphics();

		// use the paint method to draw to the off screen image
		paint(offScreenGraphics);

		// splat!! the new image over the current image
		g.drawImage(OffScreenImage, 0, 0, this);

		// release the resources associated with the off screen image
		offScreenGraphics.dispose();
	}

	public void start()
	{
		if (ImageThread == null) {
			ImageThread = new Thread(this);
			ImageThread.start();
		}
	}
	
	public void stop()
	{
		if (ImageThread != null) {
			ImageThread.stop();
			ImageThread = null;
		}
	}

	public void run()
	{
		try {
			ImageTracker.waitForID(ImageId);
		} 
		catch (InterruptedException e) {}

		ImageLoaded = true;
		ProgressMessage.hide();			// don't show progress any more

		while (true) {
			repaint();

			try {
				Thread.sleep(ImageDelay);
			} catch (InterruptedException e) {}

			CurrentDisplayXValue = GetNextXValue(CurrentDisplayXValue);
		}
	}

	protected int GetNextXValue(int currentXValue)
	{
		int nextXValue = currentXValue - ImageXDecrement;

		if (nextXValue < -ImageWidth)
			nextXValue = -ImageXDecrement;

		return nextXValue;
	}
}
